# -*- coding: utf-8 -*- 

print "Python init please wait..."

import wx
from wx import *
import gc
import wx.xrc
import wx.aui
import wx.richtext
import subprocess 
import os
import time
from numpy import *
import numpy
from traits.api import HasTraits, Instance
from traitsui.api import View, Item
from mayavi.core.ui.api import MlabSceneModel, SceneEditor
from mayavi.mlab import *
import mayavi
import urllib
import gzip
try:    
	import prody
	from prody import *
except ImportError:
    print "Prody module is required for processing pdb files! Other functionalities still available"

logger = None
logLines = 0
tabFrame = None
global OSName
global currentSettings
advancedVisualization = None

def writeLog(string,nlines,type):
# type 0 is INFO
# type 1 is WARNING
# type 2 is ERROR
	global logLines
	logLines = logLines+nlines
	if (type==0):
		logger.BeginTextColour(wx.Colour(0, 0, 0));
		logger.WriteText("\n<<INFO>> %s"%string)
	elif(type==1):
		logger.BeginTextColour(wx.Colour(0, 0, 255));
		logger.WriteText("\n<<WARNING>> %s"%string)
	elif(type==2):
		logger.BeginTextColour(wx.Colour(255, 0, 0));
		logger.WriteText("\n<<ERROR>> %s"%string)
	
	logger.ScrollLines(logLines)
	logger.Update()
	
####################################################################	
def loadOff(fileName):
	file = open(fileName,"r")	
	tag = 0
	header = 0
	numVert = 0
	numTri = 0	
	
	vert = numpy.ones(1)
	tri = numpy.ones(1)
	
	while 1:
		line = file.readline()		
		if not line:
			break
			
		if (len(line)<=2):
			continue
							
		if (tag==0 and line[0:3]=="OFF"):
			tag = 1
			continue		

		if (line[0]=='#'):			
			writeLog(("Mesh comment line: %s"%line),1,0)			
			continue
		
		# if OFF is read then the header can be read
		if (header == 0 and tag == 1):		
			v = line.split()
									
			numVert = int(v[0])
			numTri = int(v[1])
			
			if (numVert <=0 or numTri<=0):			
				writeLog("Number of triangles or vertices <=0 !",1,1)
				return 			
			else:
				vert = numpy.ones((numVert,3),dtype=float)
				tri = numpy.ones((numTri,3),dtype=int)
			header = 1;
			break;

	if (header !=1 or tag !=1):
		writeLog("Cannot read OFF header or number of vertices/triangles, stop reading",1,1)
		return
	
	# read vertices
	for i in range(numVert):
		line = file.readline()
		if (len(line)<=1):		
			i = i - 1
			continue
		v = line.split()
		
		vert[i,0] = float(v[0])
		vert[i,1] = float(v[1])
		vert[i,2] = float(v[2])		
	
	# read triangles
	for i in range(numTri):
		line = file.readline()
		if (len(line)<=1):		
			i = i - 1
			continue

		v = line.split()
		tri[i,0] = int(v[1])
		tri[i,1] = int(v[2])
		tri[i,2] = int(v[3])			

		if (int(v[0])!=3):		
			writeLog("Non triangular mesh, stop reading",1,1)
			return
				
	file.close()	
	return numVert,numTri,tri,vert
	
def showCavities():

	vec = numpy.fromfile('cavities.txt', sep=' ', dtype=double)		
	if (len(vec)==0):
		return

	vec2 = vec.reshape(len(vec)/3,3)
	points3d(vec2[:,0],vec2[:,1],vec2[:,2],color=(1.0,0.0,0.0),scale_factor=.15)
	
	vec = numpy.fromfile('cavitiesSize.txt', sep=' ', dtype=double)	
	ncav = int(vec[0])
	vec2 = vec[1:len(vec)]
	vec3 = vec2.reshape(ncav,5)	
	for i in range(ncav):		
		theLabel = '---------->> (%.1f)'%(vec3[i,3]/11.4)		
		if (vec3[i,4]==1):			
			theLabel = theLabel+' (filled)'
			writeLog(('Cavity '+("%d"%i)+theLabel),1,0)
			text3d(vec3[i,0],vec3[i,1],vec3[i,2],theLabel,scale=0.20,color=(0.0,0.0,1.0))			
		else:			
			writeLog(('Cavity '+("%d"%i)+theLabel),1,0)
			text3d(vec3[i,0],vec3[i,1],vec3[i,2],theLabel,scale=0.20,color=(0.0,0.0,0.0))
	
	
class Settings():
	Grid_scale = '2.0'
	# Percentage that the surface occupies with respect to the total grid volume 
	Grid_perfil = '90.0'
	# Input atoms xyzr file name
	XYZR_FileName = ""
	################### Surface Type params ################################
	# Possible values: msms,mesh,skin,blobby,ses
	Surface = "ses"
	Smooth_Mesh = 'true'
	Surface_File_Name = ""
	# Skin surface parameter
	Skin_Surface_Parameter = '0.45'
	# Blobbyness value for the blobby surface
	Blobbyness = '-2.5'

	# indicates the maximal number of triangles per auxiliary grid cells. Default value is 250.
	# Use bigger values for very large mesh
	Max_triangles_per_auxiliary_grid_cell = '250'
	########################################################################

	######################## Surface Processing params ###############################
	# Enable or disable cavity detection together with the volume conditional
	# filling of voids and cavities
	Cavity_Detection_Filling = 'false'

	# It is the value of the minimal volume of a cavity to get filled if cavity detection
	# is enabled. The default value is an approximation of the volume of the water molecule 
	Conditional_Volume_Filling_Value = '11.4'

	Keep_Water_Shaped_Cavities = 'false'
	Probe_Radius = '1.4'

	Accurate_Triangulation = 'true'

	# Enable triangulation: the execution time will be doubled due to the doubled number of shot rays
	Triangulation = 'true'

	# Check duplicated vertices both when writing the mesh (marching cubes) or reading
	Check_duplicated_vertices = 'true'

	# If true save the status map. Enable this for cavity detection results visualization
	Save_Status_map = 'false'
	
	Save_PovRay = 'false'
	
	# Number of threads
	Number_thread = '64'

	#############################################################################
	Project_boundary_grid_points = 'false'

	def getOptions(self):
		opt = {}
		opt['Grid_scale'] = self.Grid_scale		
		opt['Grid_perfil'] = self.Grid_perfil	
		opt['XYZR_FileName'] = self.XYZR_FileName
		opt['Surface'] = self.Surface
		opt['Smooth_Mesh'] = self.Smooth_Mesh
		opt['Surface_File_Name'] = self.Surface_File_Name	
		opt['Skin_Surface_Parameter'] = self.Skin_Surface_Parameter	
		opt['Blobbyness'] = self.Blobbyness
		opt['Max_triangles_per_auxiliary_grid_cell'] = self.Max_triangles_per_auxiliary_grid_cell
		opt['Cavity_Detection_Filling'] = self.Cavity_Detection_Filling
		opt['Conditional_Volume_Filling_Value'] = self.Conditional_Volume_Filling_Value
		opt['Keep_Water_Shaped_Cavities'] = self.Keep_Water_Shaped_Cavities
		opt['Probe_Radius'] = self.Probe_Radius
		opt['Accurate_Triangulation'] = self.Accurate_Triangulation
		opt['Triangulation'] = self.Triangulation
		opt['Check_duplicated_vertices'] = self.Check_duplicated_vertices
		opt['Save_Status_map'] = self.Save_Status_map				
		opt['Save_PovRay'] = self.Save_PovRay
		opt['Number_thread'] = self.Number_thread
		return opt
		
	def setOptions(self,opt):		
		self.Grid_scale = opt['Grid_scale']		
		self.Grid_perfil = opt['Grid_perfil']	
		self.XYZR_FileName = opt['XYZR_FileName']
		self.Surface = opt['Surface']
		self.Smooth_Mesh = opt['Smooth_Mesh']
		self.Surface_File_Name = opt['Surface_File_Name']
		self.Skin_Surface_Parameter	= opt['Skin_Surface_Parameter']
		self.Blobbyness = opt['Blobbyness']
		self.Max_triangles_per_auxiliary_grid_cell = opt['Max_triangles_per_auxiliary_grid_cell']
		self.Cavity_Detection_Filling = opt['Cavity_Detection_Filling']
		self.Conditional_Volume_Filling_Value = opt['Conditional_Volume_Filling_Value']
		self.Keep_Water_Shaped_Cavities = opt['Keep_Water_Shaped_Cavities']
		self.Probe_Radius = opt['Probe_Radius']
		self.Accurate_Triangulation = opt['Accurate_Triangulation']
		self.Triangulation = opt['Triangulation']
		self.Check_duplicated_vertices = opt['Check_duplicated_vertices']
		self.Save_Status_map = opt['Save_Status_map']
		self.Save_PovRay = opt['Save_PovRay']
		self.Number_thread = opt['Number_thread']

###########################################################################
## Class MainFrame
###########################################################################

class MainFrame ( wx.Frame ):
	
	def __init__( self, parent ):
		wx.Frame.__init__ ( self, parent, id = wx.ID_ANY, title = u"NanoShaper Viewer", pos = wx.DefaultPosition, size = wx.Size( 800,600 ), style = wx.DEFAULT_FRAME_STYLE|wx.TAB_TRAVERSAL )
		
		self.SetSizeHintsSz( wx.Size( 800,600 ), wx.DefaultSize )
		self.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_WINDOW ) )
		
		self.m_menubar1 = wx.MenuBar( 0 )
		self.m_menu1 = wx.Menu()
				
		self.m_menuItem9 = wx.MenuItem( self.m_menu1, wx.ID_ANY, u"Load pdb", wx.EmptyString, wx.ITEM_NORMAL )
		self.m_menu1.AppendItem( self.m_menuItem9 )
		
		self.m_menubar1.Append( self.m_menu1, u"&File" ) 
		
		self.m_menu8 = wx.Menu()
		self.m_menuItem11 = wx.MenuItem( self.m_menu8, wx.ID_ANY, u"Clean Console", wx.EmptyString, wx.ITEM_NORMAL )
		self.m_menu8.AppendItem( self.m_menuItem11 )
		
		self.m_menubar1.Append( self.m_menu8, u"View" ) 
		
		self.m_menu2 = wx.Menu()
		self.m_menuItem3 = wx.MenuItem( self.m_menu2, wx.ID_ANY, u"Settings", wx.EmptyString, wx.ITEM_NORMAL )
		self.m_menu2.AppendItem( self.m_menuItem3 )
		
		self.m_menubar1.Append( self.m_menu2, u"Configuration" ) 
		
		self.m_menu4 = wx.Menu()
		self.m_menuItem7 = wx.MenuItem( self.m_menu4, wx.ID_ANY, u"Run", wx.EmptyString, wx.ITEM_NORMAL )
		self.m_menu4.AppendItem( self.m_menuItem7 )
		
		self.m_menubar1.Append( self.m_menu4, u"Exec" ) 
		
		self.m_menu3 = wx.Menu()
		self.m_menuItem8 = wx.MenuItem( self.m_menu3, wx.ID_ANY, u"About", wx.EmptyString, wx.ITEM_NORMAL )
		self.m_menu3.AppendItem( self.m_menuItem8 )
		
		self.m_menubar1.Append( self.m_menu3, u"Help" ) 
		
		self.SetMenuBar( self.m_menubar1 )
		
		bSizer8 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_toolBar1 = wx.ToolBar( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TB_HORIZONTAL ) 
				
		t3 = self.m_toolBar1.AddLabelTool( wx.ID_ANY, u"tool", wx.Bitmap( u"icons/32x32/database_save.png", wx.BITMAP_TYPE_ANY ), wx.NullBitmap, wx.ITEM_NORMAL, u"Load/Download PDB", wx.EmptyString, None ) 
		
		t4 = self.m_toolBar1.AddLabelTool( wx.ID_ANY, u"tool", wx.Bitmap( u"icons/32x32/draw_eraser.png", wx.BITMAP_TYPE_ANY ), wx.NullBitmap, wx.ITEM_NORMAL, u"Clear log window", wx.EmptyString, None ) 
		
		t5 = self.m_toolBar1.AddLabelTool( wx.ID_ANY, u"tool", wx.Bitmap( u"icons/32x32/cog.png", wx.BITMAP_TYPE_ANY ), wx.NullBitmap, wx.ITEM_NORMAL, u"Configuration", wx.EmptyString, None ) 
		
		t6 = self.m_toolBar1.AddLabelTool( wx.ID_ANY, u"tool", wx.Bitmap( u"icons/32x32/cube_molecule.png", wx.BITMAP_TYPE_ANY ), wx.NullBitmap, wx.ITEM_NORMAL, u"Run NanoShaper", wx.EmptyString, None ) 
		
		self.m_toolBar1.Realize() 
		
		bSizer8.Add( self.m_toolBar1, 0, wx.EXPAND, 5 )
		
		self.m_auinotebook2 = wx.aui.AuiNotebook( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.aui.AUI_NB_DEFAULT_STYLE )
		
		bSizer8.Add( self.m_auinotebook2, 1, wx.EXPAND |wx.ALL, 5 )
		
		sbSizer15 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Log Window" ), wx.VERTICAL )
		
		sbSizer15.SetMinSize( wx.Size( -1,150 ) ) 
		self.m_richText2 = wx.richtext.RichTextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.Size( -1,150 ), wx.TE_READONLY|wx.HSCROLL|wx.NO_BORDER|wx.VSCROLL|wx.WANTS_CHARS )
		self.m_richText2.SetFont( wx.Font( 8, 74, 90, 90, False, "Consolas" ) )
		self.m_richText2.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_SCROLLBAR ) )
		self.m_richText2.SetMinSize( wx.Size( -1,150 ) )
		
		sbSizer15.Add( self.m_richText2, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALL|wx.EXPAND, 5 )
		
		bSizer8.Add( sbSizer15, 0, wx.EXPAND|wx.ALL, 5 )
		
		self.SetSizer( bSizer8 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.Bind( wx.EVT_MENU, self.OpenPDBFrame, id = self.m_menuItem9.GetId() )
		self.Bind( wx.EVT_MENU, self.CleanConsole, id = self.m_menuItem11.GetId() )
		self.Bind( wx.EVT_MENU, self.ShowSettings, id = self.m_menuItem3.GetId() )
		self.Bind( wx.EVT_MENU, self.RunEngine, id = self.m_menuItem7.GetId() )
		self.Bind( wx.EVT_MENU, self.ShowAbout, id = self.m_menuItem8.GetId() )
		
		self.Bind( wx.EVT_TOOL, self.OpenPDBFrame, id = t3.GetId() )
		self.Bind( wx.EVT_TOOL, self.CleanConsole, id = t4.GetId() )
		self.Bind( wx.EVT_TOOL, self.ShowSettings, id = t5.GetId() )
		self.Bind( wx.EVT_TOOL, self.RunEngine, id = t6.GetId() )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	
	def OpenPDBFrame( self, event ):
		pass
	
	def CleanConsole( self, event ):
		pass
	
	def ShowSettings( self, event ):
		pass
	
	def RunEngine( self, event ):
		pass
	
	def ShowAbout( self, event ):
		pass

###########################################################################
## Class ConfFrame
###########################################################################

class ConfFrame ( wx.Frame ):
	
	def __init__( self, parent ):
		wx.Frame.__init__ ( self, parent, id = wx.ID_ANY, title = u"Configuration", pos = wx.DefaultPosition, size = wx.Size( 652,573 ), style = wx.DEFAULT_FRAME_STYLE|wx.TAB_TRAVERSAL )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		self.SetBackgroundColour( wx.Colour( 255, 255, 255 ) )
		
		bSizer4 = wx.BoxSizer( wx.VERTICAL )
		
		gSizer3 = wx.GridSizer( 0, 2, 0, 0 )
		
		sbSizer12 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Molecue (.xyzr)" ), wx.VERTICAL )
		
		self.m_filePicker3 = wx.FilePickerCtrl( self, wx.ID_ANY, wx.EmptyString, u"Select mol file", u"*.xyzr", wx.DefaultPosition, wx.DefaultSize, wx.FLP_DEFAULT_STYLE )
		sbSizer12.Add( self.m_filePicker3, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL|wx.TOP|wx.RIGHT|wx.LEFT, 5 )
		
		gSizer3.Add( sbSizer12, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		sbSizer13 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Input Mesh File" ), wx.VERTICAL )
		
		self.m_filePicker2 = wx.FilePickerCtrl( self, wx.ID_ANY, wx.EmptyString, u"Select Input Mesh File", u"*.off;*.ply;*.vert;*.face", wx.DefaultPosition, wx.DefaultSize, wx.FLP_DEFAULT_STYLE )
		sbSizer13.Add( self.m_filePicker2, 0, wx.ALIGN_CENTER|wx.ALL, 5 )
		
		gSizer3.Add( sbSizer13, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_CENTER_HORIZONTAL|wx.EXPAND, 5 )
		
		bSizer4.Add( gSizer3, 0, wx.EXPAND, 5 )
		
		sbSizer1 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Grid Settings" ), wx.VERTICAL )
		
		gSizer2 = wx.GridSizer( 0, 2, 0, 0 )
		
		sbSizer10 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Grid Scale (A)" ), wx.VERTICAL )
		
		self.m_textCtrl4 = wx.TextCtrl( self, wx.ID_ANY, u"2.0", wx.DefaultPosition, wx.DefaultSize, 0 )
		sbSizer10.Add( self.m_textCtrl4, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		gSizer2.Add( sbSizer10, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		sbSizer11 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Perfil (%)" ), wx.VERTICAL )
		
		self.m_textCtrl5 = wx.TextCtrl( self, wx.ID_ANY, u"80", wx.DefaultPosition, wx.DefaultSize, 0 )
		sbSizer11.Add( self.m_textCtrl5, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		gSizer2.Add( sbSizer11, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		sbSizer1.Add( gSizer2, 0, wx.EXPAND, 5 )
		
		bSizer4.Add( sbSizer1, 0, wx.EXPAND, 5 )
		
		sbSizer3 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Surface Settings" ), wx.VERTICAL )
		
		gSizer1 = wx.GridSizer( 0, 2, 0, 0 )
		
		sbSizer5 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Surface Type" ), wx.VERTICAL )
		
		m_comboBox1Choices = [ u"ses", u"skin", u"blobby", u"mesh", u"msms" ]
		self.m_comboBox1 = wx.ComboBox( self, wx.ID_ANY, u"SES", wx.Point( 60,20 ), wx.DefaultSize, m_comboBox1Choices, 0 )
		sbSizer5.Add( self.m_comboBox1, 0, wx.ALIGN_CENTER, 5 )
		
		gSizer1.Add( sbSizer5, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		sbSizer6 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Probe Radius" ), wx.VERTICAL )
		
		self.m_textCtrl1 = wx.TextCtrl( self, wx.ID_ANY, u"1.4", wx.DefaultPosition, wx.DefaultSize, 0 )
		sbSizer6.Add( self.m_textCtrl1, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		gSizer1.Add( sbSizer6, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		sbSizer7 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Skin parameter" ), wx.VERTICAL )
		
		self.m_textCtrl2 = wx.TextCtrl( self, wx.ID_ANY, u"0.45", wx.DefaultPosition, wx.DefaultSize, 0 )
		sbSizer7.Add( self.m_textCtrl2, 0, wx.ALL, 5 )
		
		gSizer1.Add( sbSizer7, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		sbSizer8 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Blobbyness" ), wx.VERTICAL )
		
		self.m_textCtrl3 = wx.TextCtrl( self, wx.ID_ANY, u"-2.5", wx.DefaultPosition, wx.DefaultSize, 0 )
		sbSizer8.Add( self.m_textCtrl3, 0, wx.ALL, 5 )
		
		gSizer1.Add( sbSizer8, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_checkBox1 = wx.CheckBox( self, wx.ID_ANY, u"Mesh Smoothing", wx.DefaultPosition, wx.DefaultSize, 0 )
		gSizer1.Add( self.m_checkBox1, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.TOP|wx.BOTTOM|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		self.m_checkBox2 = wx.CheckBox( self, wx.ID_ANY, u"Accurate Triangulation", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_checkBox2.SetValue(True) 
		gSizer1.Add( self.m_checkBox2, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		self.m_checkBox3 = wx.CheckBox( self, wx.ID_ANY, u"Cavity Detection", wx.DefaultPosition, wx.DefaultSize, 0 )
		gSizer1.Add( self.m_checkBox3, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		sbSizer14 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Filter Cavities less than (A^3)" ), wx.VERTICAL )
		
		self.m_textCtrl6 = wx.TextCtrl( self, wx.ID_ANY, u"11.4", wx.DefaultPosition, wx.DefaultSize, 0 )
		sbSizer14.Add( self.m_textCtrl6, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		gSizer1.Add( sbSizer14, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		sbSizer3.Add( gSizer1, 0, wx.EXPAND, 5 )
		
		bSizer4.Add( sbSizer3, 0, wx.EXPAND, 5 )
		
		sbSizer2 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Other Options" ), wx.VERTICAL )
		
		gSizer4 = wx.GridSizer( 0, 2, 0, 0 )
		
		self.m_checkBox4 = wx.CheckBox( self, wx.ID_ANY, u"Save Status Map", wx.DefaultPosition, wx.DefaultSize, 0 )
		gSizer4.Add( self.m_checkBox4, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.m_checkBox5 = wx.CheckBox( self, wx.ID_ANY, u"Write PovRay Surf", wx.DefaultPosition, wx.DefaultSize, 0 )
		gSizer4.Add( self.m_checkBox5, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		sbSizer15 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Execution Threads" ), wx.VERTICAL )
		
		self.m_textCtrl7 = wx.TextCtrl( self, wx.ID_ANY, u"32", wx.DefaultPosition, wx.DefaultSize, 0 )
		sbSizer15.Add( self.m_textCtrl7, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALL, 5 )
		
		gSizer4.Add( sbSizer15, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		sbSizer2.Add( gSizer4, 1, wx.EXPAND, 5 )
		
		bSizer4.Add( sbSizer2, 0, wx.EXPAND, 5 )
		
		m_sdbSizer1 = wx.StdDialogButtonSizer()
		self.m_sdbSizer1OK = wx.Button( self, wx.ID_OK )
		m_sdbSizer1.AddButton( self.m_sdbSizer1OK )
		self.m_sdbSizer1Cancel = wx.Button( self, wx.ID_CANCEL )
		m_sdbSizer1.AddButton( self.m_sdbSizer1Cancel )
		m_sdbSizer1.Realize();
		m_sdbSizer1.SetMinSize( wx.Size( 50,50 ) ) 
		bSizer4.Add( m_sdbSizer1, 0, wx.ALIGN_CENTER|wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.SetSizer( bSizer4 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.m_sdbSizer1Cancel.Bind( wx.EVT_BUTTON, self._close )
		self.m_sdbSizer1OK.Bind( wx.EVT_BUTTON, self.getConfiguration )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def _close( self, event ):
		pass
	
	def getConfiguration( self, event ):
		pass
	
class ConfFrameD ( ConfFrame ):		

	def __init__( self, parent ):
		
		super(ConfFrameD,self).__init__(parent)
		
		self.m_filePicker3.SetPath(currentSettings.XYZR_FileName)
		self.m_filePicker2.SetPath(currentSettings.Surface_File_Name)
		self.m_textCtrl4.SetValue(currentSettings.Grid_scale)
		self.m_textCtrl5.SetValue(currentSettings.Grid_perfil)
		self.m_comboBox1.SetValue(currentSettings.Surface)
		self.m_textCtrl1.SetValue(currentSettings.Probe_Radius)
		self.m_textCtrl2.SetValue(currentSettings.Skin_Surface_Parameter)
		self.m_textCtrl3.SetValue(currentSettings.Blobbyness)
				
		if (currentSettings.Smooth_Mesh=='true'):
			self.m_checkBox1.SetValue(True)
		else:
			self.m_checkBox1.SetValue(False)
		
		if (currentSettings.Accurate_Triangulation=='true'):
			self.m_checkBox2.SetValue(True)
		else:
			self.m_checkBox2.SetValue(False)

		if (currentSettings.Cavity_Detection_Filling=='true'):
			self.m_checkBox3.SetValue(True)
		else:
			self.m_checkBox3.SetValue(False)
			
		self.m_textCtrl6.SetValue(currentSettings.Conditional_Volume_Filling_Value)
		
		if (currentSettings.Save_Status_map=='true'):
			self.m_checkBox4.SetValue(True)
		else:
			self.m_checkBox4.SetValue(False)
		
		if (currentSettings.Save_PovRay=='true'):
			self.m_checkBox5.SetValue(True)
		else:
			self.m_checkBox5.SetValue(False)
			
		self.m_textCtrl7.SetValue(currentSettings.Number_thread)
			
	def getConfiguration( self, event ):
	
		global currentSettings
		
		currentSettings.XYZR_FileName =  self.m_filePicker3.GetPath().encode('utf8')
		currentSettings.Surface_File_Name =  self.m_filePicker2.GetPath().encode('utf8')
		currentSettings.Grid_scale = self.m_textCtrl4.GetValue().encode('utf8')
		currentSettings.Grid_perfil = self.m_textCtrl5.GetValue().encode('utf8')
		currentSettings.Surface = self.m_comboBox1.GetValue().encode('utf8')
		currentSettings.Probe_Radius = self.m_textCtrl1.GetValue().encode('utf8')
		currentSettings.Skin_Surface_Parameter = self.m_textCtrl2.GetValue().encode('utf8')
		currentSettings.Blobbyness = self.m_textCtrl3.GetValue().encode('utf8')
		
		state = self.m_checkBox1.GetValue()
		if (state==True):
			currentSettings.Smooth_Mesh = 'true'
		else:
			currentSettings.Smooth_Mesh = 'false'
		
		state = self.m_checkBox2.GetValue()
		if (state==True):
			currentSettings.Accurate_Triangulation = 'true'
		else:
			currentSettings.Accurate_Triangulation = 'false'
		
		state = self.m_checkBox3.GetValue()
		if (state==True):
			currentSettings.Cavity_Detection_Filling = 'true'
		else:
			currentSettings.Cavity_Detection_Filling= 'false'
			
		currentSettings.Conditional_Volume_Filling_Value = self.m_textCtrl6.GetValue().encode('utf8')
		
		state = self.m_checkBox4.GetValue()
		if (state==True):
			currentSettings.Save_Status_map = 'true'
		else:
			currentSettings.Save_Status_map = 'false'
			
		state = self.m_checkBox5.GetValue()
		if (state==True):
			currentSettings.Save_PovRay = 'true'
		else:
			currentSettings.Save_PovRay = 'false'
			
		currentSettings.Number_thread = self.m_textCtrl7.GetValue().encode('utf8')
		
		self.Close()
	
	def _close( self, event ):
		self.Close()
	

###########################################################################
## Class pdbFrame
###########################################################################

class pdbFrame ( wx.Frame ):
	
	def __init__( self, parent ):
		wx.Frame.__init__ ( self, parent, id = wx.ID_ANY, title = u"Pdb Processor", pos = wx.DefaultPosition, size = wx.Size( 400,180 ), style = wx.DEFAULT_FRAME_STYLE|wx.TAB_TRAVERSAL )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		self.SetBackgroundColour( wx.Colour( 255, 255, 255 ) )
		
		bSizer5 = wx.BoxSizer( wx.VERTICAL )
		
		gSizer5 = wx.GridSizer( 0, 2, 0, 0 )
		
		sbSizer15 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Get from disk" ), wx.VERTICAL )
		
		self.m_filePicker3 = wx.FilePickerCtrl( self, wx.ID_ANY, wx.EmptyString, u"Select a file", u"*.*", wx.DefaultPosition, wx.DefaultSize, wx.FLP_DEFAULT_STYLE )
		sbSizer15.Add( self.m_filePicker3, 1, wx.ALIGN_CENTER_HORIZONTAL|wx.EXPAND, 0 )
		
		gSizer5.Add( sbSizer15, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_CENTER_HORIZONTAL|wx.EXPAND|wx.ALL, 5 )
		
		sbSizer16 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"PDB Code (Internet)" ), wx.VERTICAL )
		
		self.m_textCtrl8 = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		sbSizer16.Add( self.m_textCtrl8, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		gSizer5.Add( sbSizer16, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 0 )
		
		bSizer5.Add( gSizer5, 0, wx.ALIGN_CENTER_HORIZONTAL, 0 )
		
		sbSizer17 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Atom Subset selection string" ), wx.VERTICAL )
		
		sbSizer17.SetMinSize( wx.Size( 300,-1 ) ) 
		self.m_textCtrl9 = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_textCtrl9.SetMinSize( wx.Size( 300,-1 ) )
		
		sbSizer17.Add( self.m_textCtrl9, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		bSizer5.Add( sbSizer17, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALL, 0 )
		
		m_sdbSizer2 = wx.StdDialogButtonSizer()
		self.m_sdbSizer2OK = wx.Button( self, wx.ID_OK )
		m_sdbSizer2.AddButton( self.m_sdbSizer2OK )
		self.m_sdbSizer2Cancel = wx.Button( self, wx.ID_CANCEL )
		m_sdbSizer2.AddButton( self.m_sdbSizer2Cancel )
		m_sdbSizer2.Realize();
		bSizer5.Add( m_sdbSizer2, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALL, 5 )
		
		self.SetSizer( bSizer5 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.m_sdbSizer2Cancel.Bind( wx.EVT_BUTTON, self._close )
		self.m_sdbSizer2OK.Bind( wx.EVT_BUTTON, self.runPDBprocessor )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def _close( self, event ):
		pass
	
	def runPDBprocessor( self, event ):
		pass
	
class pdbFrameD(pdbFrame):

	def _close( self, event ):
		self.Close(True)
		
	def runPDBprocessor( self, event):
			
		global logger 
		localFile = False
		
		pdbName = self.m_textCtrl8.GetValue()
		pdbLocal = self.m_filePicker3.GetPath().encode('utf8')
		
		if (len(pdbName)==0 and len(pdbLocal)==0):
			writeLog('Please insert PDB file or code',1,1)
			return
		
		if (len(pdbName)!=0):
			pdbName = pdbName.encode('utf8')
			writeLog('Pdb name: %s'%(pdbName),1,0)
			if (len(pdbName)!=4):
				writeLog('Pdb should be 4 characters',1,1)
				self.Close(True)
				return 
			
			dir = pdbName[1:3]		
			# download pdb if necessary
			writeLog('Downloading pdb, please wait...',1,0)
		
			myurl = ('ftp://ftp.wwpdb.org/pub/pdb/data/structures/divided/pdb/%s/pdb%s.ent.gz'%(dir,pdbName))
			opener = urllib.urlopen(myurl)
			open('pdb%s.ent.gz' %pdbName, 'wb').write(opener.read())
			infile = gzip.GzipFile('pdb%s.ent.gz' %pdbName, 'rb')
		
			f = open(("%s.pdb"%pdbName),'w')
			for line in infile:
				f.write(line)
		
			f.close()		
			infile.close()
		
			writeLog('Dowload completed',1,0)
			writeLog(('File saved in %s.pdb'%(pdbName)),1,0)
		else:
			writeLog('Reading %s from local unit'%pdbLocal,1,0)
			pdbName = pdbLocal[0:len(pdbLocal)-4]
			localFile = True
				
		pdbSys = prody.parsePDB(('%s.pdb'%pdbName))
		
		selection = self.m_textCtrl9.GetValue().encode('utf8')
		
		if (len(selection)!=0):
			pdbSys = pdbSys.select(selection)
			writeLog('Applying selection: "%s"'%selection,1,0)
						
			write_file = ''
			if (localFile):				
				splitChar = ''
				if (OSName == 'nt'):
					splitChar = '\\'					
				else:
					splitChar = '/'
				
				name = pdbName.split(splitChar)	
			
				for i in range(len(name)-1):
					write_file=write_file+name[i]+splitChar					
					
				tmp = name[len(name)-1]
				write_file = write_file+'selection_'+tmp
			else:
				write_file = 'selection_%s.pdb'%pdbName
			
			prody.writePDB(write_file,pdbSys.select(selection))			
			pdbSys = prody.parsePDB(write_file)
				
		FFfileName = 'amber.siz'
		
		# load force field parameters
		fileFF = open(FFfileName,'r')
		
		defaultRadii = {}
		specificRadii = {}
		lineNum = 0
		while 1:
			line = fileFF.readline()		
			lineNum = lineNum+1
			
			# stop reading
			if not line:
				break
			
			# skip empty lines
			if (len(line)<=1):				
				continue
			
			# skip comment line
			if '!' in line:	
				continue
			
			if (line.startswith('atom__res_radius_')):
				writeLog('Recognized DelPhi header for radii .siz file',1,0)				
				continue
						
			listStr = line.split()
			if (len(listStr)==2):
				defaultRadii[listStr[0][0]]=float(listStr[1])
			elif (len(listStr)==3):
				specificRadii['%s_%s'%(listStr[0],listStr[1])]=float(listStr[2])
			else:
				writeLog('Unrecognized record in radii file at line %d, please check file and reload'%lineNum,1,1)
				return
						
												
		f = open(('%s.xyzr'%pdbName),'w')
				
		X = pdbSys.getCoords()
		
		writeLog('Converting PDB using %s'%FFfileName,1,0)
		for i in range(len(pdbSys)):
			resName = pdbSys[i].getResname()
			atomName = pdbSys[i].getName()
			key = '%s_%s'%(atomName,resName)
			if not(key in specificRadii):
				if not(atomName[0] in defaultRadii):
					writeLog('%s %s missing in radii set applying 1.0 Angstrom'%(atomName,resName),1,1)
					radius = 1.0
				else:
					radius = defaultRadii[atomName[0]]
			else:
				radius = specificRadii[key]
			
			f.write(('%f %f %f %f\n'%(X[i][0],X[i][1],X[i][2],radius)))
		f.close()		
			
		writeLog('File converted in %s.xyzr'%(pdbName),1,0)
		writeLog('Current mol file set to %s.xyzr'%(pdbName),1,0)
		
		currentSettings.XYZR_FileName = '%s.xyzr'%pdbName
				
		self.Close(True)
		

###########################################################################
## Class AboutFrame
###########################################################################

class AboutFrame ( wx.Frame ):
	
	def __init__( self, parent ):
		wx.Frame.__init__ ( self, parent, id = wx.ID_ANY, title = u"About", pos = wx.DefaultPosition, size = wx.Size( 207,118 ), style = wx.DEFAULT_FRAME_STYLE|wx.TAB_TRAVERSAL )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		self.SetBackgroundColour( wx.Colour( 255, 255, 255 ) )
		
		bSizer1 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_panel9 = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		self.m_panel9.SetBackgroundColour( wx.Colour( 255, 255, 255 ) )
		
		bSizer5 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_staticText2 = wx.StaticText( self.m_panel9, wx.ID_ANY, u"NSgui version 0.1 \nAuthor: Sergio Decherchi \nContact: sergio.decherchi@iit.it", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText2.Wrap( -1 )
		self.m_staticText2.SetBackgroundColour( wx.Colour( 255, 255, 255 ) )
		
		bSizer5.Add( self.m_staticText2, 0, wx.ALL, 5 )
		
		self.m_hyperlink1 = wx.HyperlinkCtrl( self.m_panel9, wx.ID_ANY, u"http://www.electrostaticszone.eu", u"http://www.electrostaticszone.eu", wx.DefaultPosition, wx.DefaultSize, wx.HL_DEFAULT_STYLE )
		self.m_hyperlink1.SetBackgroundColour( wx.Colour( 255, 255, 255 ) )
		
		bSizer5.Add( self.m_hyperlink1, 0, wx.ALL, 5 )
		
		self.m_panel9.SetSizer( bSizer5 )
		self.m_panel9.Layout()
		bSizer5.Fit( self.m_panel9 )
		bSizer1.Add( self.m_panel9, 1, wx.EXPAND |wx.ALL, 5 )
		
		self.SetSizer( bSizer1 )
		self.Layout()
		
		self.Centre( wx.BOTH )
				
	def __del__( self ):
		pass

		
class MayaviView(HasTraits):

	scene = Instance(MlabSceneModel, ())
	global logLines 
	
	# The layout of the panel created by traits.
	view = View(Item('scene', editor=SceneEditor(),
                    resizable=True,
                    show_label=False),
					resizable=True)

	def __init__(self):			
		HasTraits.__init__(self)

	def go(self):		
		nvert,ntri,tri,vert = loadOff("triangulatedSurf.off")	
				
		writeLog("Loading mesh visualization...",1,0)				
		if (advancedVisualization == True):
			writeLog('Opening Advanced Visualization',1,0)
			mayavi.mlab.options.backend = 'envisage'
		else:
			writeLog('Opening Embedded Visualization',1,0)
			
		if (currentSettings.Cavity_Detection_Filling=='true' and currentSettings.Save_Status_map=='true'):
			mesh = triangular_mesh(vert[:,0], vert[:,1], vert[:,2], tri,color=(51/255.0,204/255.0,204/255.0),opacity=0.2)
		else:	
			mesh = triangular_mesh(vert[:,0], vert[:,1], vert[:,2], tri,color=(51/255.0,204/255.0,204/255.0),opacity=1.0)
		
		if (advancedVisualization == False):
			self.scene.light_manager.light_mode = 'vtk'
			self.scene.background =(0,0,0)
			
		writeLog("Mesh loaded",1,0)						
		if (currentSettings.Cavity_Detection_Filling=='true' and currentSettings.Save_Status_map=='true'):
			showCavities()		
		
###############################################################################################
	
def parseLog(fileName):
		
	file = open(fileName,"r")
	while 1:
		line = file.readline()		
						
		if not line:
			break
				
		if (len(line)<=1):
			continue
				
		strList = line.split()
		
		if (len(strList)>=30):
			continue
		
		if (strList[0]=='<<INFO>>' and strList[1]=='MC' and strList[2]=='Completion'):
			continue
			
		if (strList[0]=='<<INFO>>' and strList[1]=='Blobby'):
			continue
			
		if (strList[0].startswith('<<INFO>>')):
			if (line[len(line)-1]=='\n'):
				writeLog(line[10:len(line)-1],1,0)
			else:
				writeLog(line[10:len(line)],1,0)
		elif(strList[0].startswith('<<WARNING>>')):
			if (line[len(line)-1]=='\n'):
				writeLog(line[13:len(line)-1],1,1)
			else:
				writeLog(line[13:len(line)],1,1)
		elif(strList[0].startswith('<<ERROR>>')):
			if (line[len(line)-1]=='\n'):
				writeLog(line[11:len(line)-1],1,2)
			else:
				writeLog(line[11:len(line)],1,2)
		else:
			if (line[len(line)-1]=='\n'):
				writeLog(line[0:len(line)-1],1,0)			
			else:
				writeLog(line[0:len(line)],1,0)			
					
	file.close()
		
	
		
class MainFrameD (MainFrame):
		
	def __init__( self, parent ):
	
		super(MainFrameD,self).__init__(parent)
		
		global logger 
		global tabFrame		
		
		logger = self.m_richText2
		tabFrame = self.m_auinotebook2
			
		pp = wx.Panel(tabFrame)
				
	def RunEngine(self,event):
				
		global advancedVisualization
		global currentSettings
		gc.collect()
		
		write_config('tempconf.prm',currentSettings.getOptions())
		writeLog("Running NanoShaper please wait...",1,0)
		
		if (OSName == 'nt'):
			subprocess.call("NanoShaper tempconf.prm > log.txt",shell=True)
		else:
			subprocess.call("./NanoShaper tempconf.prm > log.txt",shell=True)
						
		parseLog("./log.txt")				
				
		advancedVisualization = False
		
		self.surfaceView = MayaviView()
		
		if (advancedVisualization==False):
			self.control = self.surfaceView.edit_traits(
                        parent=self,
                        kind='subpanel').control
			if (OSName == 'nt'):
				name = currentSettings.XYZR_FileName.split('\\')
			else:
				name = currentSettings.XYZR_FileName.split('/')
			name = name[len(name)-1]
			tabFrame.AddPage(self.control,name)  
			
		self.surfaceView.go()				
	
	def ShowAbout( self, event ):
		f = AboutFrame(self)
		f.Show(True)		

	def CleanConsole( self, event ):
		logger.Clear()
		
	def OpenPDBFrame( self, event ):
		f = pdbFrameD(self)
		f.Show(True)
		
	def ShowSettings(self,event):
		f = ConfFrameD(self)
		f.Show(True)
		
	def LoadConf( self, event ):
			
		fp = wx.FilePicker( self, wx.ID_ANY, wx.EmptyString, u"Select config file", u"*.prm", wx.DefaultPosition, wx.DefaultSize, wx.FLP_DEFAULT_STYLE )
		fp.Show(True)		
		file = fp.GetPath()
		opt = parse_config(file)
		currentSettings.setOptions(opt)
	
	def SaveCurrentConf( self, event ):
		pass
		

############# file parser ####################################
COMMENT_CHAR = '#'
OPTION_CHAR =  '='
 
def parse_config(filename):
    options = {}
    f = open(filename)
    for line in f:
        # First, remove comments:
        if COMMENT_CHAR in line:
            # split on comment char, keep only the part before
            line, comment = line.split(COMMENT_CHAR, 1)
        # Second, find lines with an option=value:
        if OPTION_CHAR in line:
            # split on option char:
            option, value = line.split(OPTION_CHAR, 1)
            # strip spaces:
            option = option.strip()
            value = value.strip()
            # store in dictionary:
            options[option] = value
    f.close()
    return options
	
def write_config(filename,options):	
	f = open(filename,'w')
	for k, v in options.iteritems():
		f.write(('%s = %s\n'%(k,v)))
	f.close()
########################################################### 
		
if __name__ == '__main__':

	gc.enable()
	OSName = os.name
	currentSettings = Settings()
	app = wx.App()
	frame = MainFrameD(None)		
	frame.Show(True)
	app.MainLoop()
		
	
		
